/*
Laser-Tag "CS-IRL-USP"
VERSION 0.86
[c]2012-2013 DCR Dirk Conrad

DDR[x]  ... Datenrichtung (0=Eingang; 1 = Ausgang)
PORT[x] ... Ausgabe auf Port oder 1= PullUp Widerstand bei Datenrichtung einlesen
PIN[x]  ... Einlesen vom Port
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <compat/ina90.h>
#include <stdio.h>
#include <avr/pgmspace.h>

// Def
#define	WHITE_LED		0x01
#define WHITE_LED_MASK	0xFE
#define	TEAM_RED		0x02
#define	TEAM_GREEN		0x04
#define TEAM_DEATHMATCH	0x06
#define TEAM_MASK		0xF9
#define	KEY_ACTION		0x08
#define	KEY_RELOAD		0x10
#define	KEY_FIRE		0x20
#define	RECEIVE			0x40
#define RECEIVE_MASK	0xBF
#define	IR_LED			0x80
#define IR_LED_MASK		0x7F
#define	AMMO1			0x01
#define	AMMO10			0x02
#define AMMO_MASK		0xFC
#define SEG_MASK		0x7F
#define	RED1_LED		0x04
#define	RED2_LED		0x08
#define	RED3_LED		0x10
#define RED_MASK		0x1C
#define NUM_0			0x40
#define NUM_1			0x79
#define NUM_2			0x24
#define NUM_3			0x30
#define NUM_4			0x19
#define NUM_5			0x12
#define NUM_6			0x02
#define NUM_7			0x78
#define NUM_8			0x00
#define NUM_9			0x10
#define NUM_BLANK		0x7F
#define NUM_HYPHEN		0x3F
#define NUM_d			0x21
#define NUM_r			0x2F
#define NUM_A			0x08
#define NUM_C			0x46
#define NUM_E			0x06
#define NUM_F			0x0E
#define NUM_L			0x47
#define NUM_P			0x0C
#define NUM_U			0x41
#define MAX_SEND_BITS	5
#define MAX_REC_BITS	4

// COMPILE WITH SOUND: 0=DISABLE, 1=ENABLE
#define SOUND_ENABLE	1

// COMPILE FOR HARDWARE: 0=USP, 1=C4
#define HARDWARE		1

// Sub
void Init (void);
void Intro (void);
void Select_Charge (void);
void Val_To_Seg (double value);
unsigned char Get_Key (void);
void Fire (void);
#if (HARDWARE == 0)
void Set_Ammo (void);
#endif
void Chk_Receive (void);
void Sound (void);

// Var IR Send Receive
volatile unsigned int ir_led_counter = 0;
volatile unsigned int ir_rec_counter = 999;
volatile unsigned char ir_low_rising = 0;
// START BIT (1)
// TEAM (0=COMMAND, 1=RED, 2=GREEN, 3=DEATHMATCH)
// ACTION (0=DISARM/PLANT C4 1=FIRE 2=-- 3=--)
// CHECKSUM
// STOP BIT (1) 
volatile unsigned char send_buffer[] = { 1, 0, 0, 0, 1 };
volatile unsigned int send_pointer = 255;
volatile unsigned int send_pause = 0;
volatile unsigned int hit_pause = 0;
volatile unsigned char send_repeat = 0;
volatile unsigned char rec_buffer[] = { 0, 0, 0, 0 };
volatile unsigned int rec_pointer = 0;
volatile unsigned char rec_flag = 0;
volatile unsigned char timer_flag = 0;

// Var 7-SEG / HP
volatile unsigned int seg_div = 0;
volatile unsigned char seg_pos = AMMO1;
volatile unsigned char seg_value[] = {NUM_BLANK, NUM_HYPHEN};
volatile unsigned char blink = 0;
volatile unsigned char blink_counter = 0;
volatile unsigned int wait_flag = 0;
volatile unsigned char red_bar = 0;
volatile unsigned int hide_counter = 1000;
volatile unsigned long busy_counter = 100000;

// Var ADC
volatile unsigned int adc_value = 0xFFFF;

// Var Generic
#if (HARDWARE == 0)
unsigned char ammo = 0;
unsigned int reload_counter = 0;
unsigned int death_counter = 0;
#endif

#if (HARDWARE == 1)
int plant_disarm_counter = 0;
double bomb_timer = 0;
int bomb_counter = 0;
#endif

unsigned int team_blink_counter = 0;
unsigned int fire_flash_counter = 0;
unsigned char team = 0;
unsigned int key_wait = 0;

// VAR Sounds
#if (SOUND_ENABLE == 1)
	volatile unsigned char snd_div = 0;
	volatile unsigned char snd_data = 0;
	#if (HARDWARE == 0)
		const char snd_fire[1863] PROGMEM = {
			#include <snd_fire.h>
		};
		volatile unsigned long snd_fire_pointer = 1863;
		volatile unsigned long snd_fire_stop = 1863;

		const char snd_autschn[5449] PROGMEM = {
			#include <snd_autschn.h>
		};
		volatile unsigned long snd_autschn_pointer = 5449;
		volatile unsigned long snd_autschn_stop = 5449;

		const char snd_button[1393] PROGMEM = {
			#include <snd_button.h>
		};
		volatile unsigned long snd_button_pointer = 1393;
		volatile unsigned long snd_button_stop = 1393;

		const char snd_clip[4958] PROGMEM = {
			#include <snd_clip.h>
		};
		volatile unsigned long snd_clip_pointer = 4958;
		volatile unsigned long snd_clip_stop = 4958;

		const char snd_empty[662] PROGMEM = {
			#include <snd_empty.h>
		};
		volatile unsigned long snd_empty_pointer = 662;
		volatile unsigned long snd_empty_stop = 662;

		const char snd_manno[5853] PROGMEM = {
			#include <snd_manno.h>
		};
		volatile unsigned long snd_manno_pointer = 5853;
		volatile unsigned long snd_manno_stop = 5853;

		const char snd_release[3635] PROGMEM = {
			#include <snd_release.h>
		};
		volatile unsigned long snd_release_pointer = 3635;
		volatile unsigned long snd_release_stop = 3635;
	#endif

	#if (HARDWARE == 1)
		const char snd_c4_beep[1090] PROGMEM = {
			#include <snd_c4_beep.h>
		};
		volatile unsigned long snd_c4_beep_pointer = 1090;
		volatile unsigned long snd_c4_beep_stop = 1090;

		const char snd_c4_explode[11996] PROGMEM = {
			#include <snd_c4_explode.h>
		};
		volatile unsigned long snd_c4_explode_pointer = 11996;
		volatile unsigned long snd_c4_explode_stop = 11996;

		const char snd_bomb_planted[12114] PROGMEM = {
			#include <snd_bomb_planted.h>
		};
		volatile unsigned long snd_bomb_planted_pointer = 12114;
		volatile unsigned long snd_bomb_planted_stop = 12114;
	#endif
#endif

// ########## Hauptprogramm ##########

int main (void) {
unsigned char key = 0;
unsigned int key_action_counter = 0;
unsigned char play_loop = 1;

	while (1) {
		Init ();
		red_bar = 0;
		team = 0;
		PORTD |= WHITE_LED;
		Intro ();

		// Team Select / Charge Accu
		Select_Charge ();

		rec_flag = 0;
		PORTD |= WHITE_LED;	

		#if (HARDWARE == 0)
		red_bar = 3;
		ammo = 0;
		reload_counter = 400;
		#endif

		#if (HARDWARE == 1)
		team = 0;
		seg_value[0] = NUM_P;
		seg_value[1] = NUM_L;	
		blink = 1;
		red_bar = 0;
		bomb_timer = 0.0;
		#endif

		// Play Loop
		play_loop = 1;
		while (play_loop) {

			key = Get_Key ();	
			
			#if (HARDWARE == 0)
			// Fire
			if (key == KEY_FIRE) {
				hide_counter = 1000;
				busy_counter = 100000;
				Fire ();
			}
			#endif

			#if (HARDWARE == 0)
			// Reload / Respawn
			if (key == KEY_RELOAD) {
				hide_counter = 1000;
				busy_counter = 100000;
				if (red_bar > 0 && ammo < 12) {
					reload_counter = 400;
					#if (SOUND_ENABLE == 1)
					snd_clip_pointer = 0;
					#endif
				}
				if (red_bar == 0 && death_counter == 0) {
					ammo = 0;
					reload_counter = 400;
					rec_flag = 0;
					red_bar = 3;
				}
			}
			#endif

			Chk_Receive ();

			if (wait_flag) {
				wait_flag = 0;

				#if (HARDWARE == 0)
				// Reload
				if (reload_counter > 0 && red_bar > 0) {
					blink = 1;
					Set_Ammo ();
					reload_counter--;
					if (reload_counter == 0 && red_bar > 0) {
						ammo = 12;
						Set_Ammo ();
						blink = 0;
						#if (SOUND_ENABLE == 1)
						snd_release_pointer = 0;
						#endif
					}
				}
				#endif

				#if (HARDWARE == 0)
				// Dead
				if (death_counter > 0) {
					death_counter--;
					hide_counter = 1000;
					busy_counter = 100000;
					if (death_counter == 0) {
						// Enable Respawn
						seg_value[0] = NUM_r;
						seg_value[1] = NUM_BLANK;
						DDRD &= RECEIVE_MASK;	
						PORTD |= RECEIVE;
					}
				}
				#endif

				// Fire Flash
				if (fire_flash_counter > 0) {
					fire_flash_counter--;
					PORTD &= WHITE_LED_MASK;
					if (fire_flash_counter == 0) {
						PORTD |= WHITE_LED;				
					}
				}

				// Key Action
				key = PIND & KEY_ACTION;
				if (key == 0) {
					#if (HARDWARE == 0)
					if (hide_counter > 0 && hide_counter < 990 && reload_counter == 0) {
						if (seg_value[0] != NUM_r && seg_value[0] != NUM_d) {
							if (seg_value[0] != NUM_C) {
								seg_value[0] = NUM_C;
								seg_value[1] = NUM_4;
							}
							else {
								Set_Ammo ();
							}
						}
					}
					#endif
					
					#if (HARDWARE == 1)
					// Plant
					if (seg_value[0] == NUM_P && red_bar == 3) {
						bomb_timer = 37.0;
						Val_To_Seg (35.0);
						red_bar = 0;
						blink = 0;
						team = 1;
						#if (SOUND_ENABLE == 1)
						snd_bomb_planted_pointer = 0;
						#endif
					}
					// Disarm
					if (seg_value[0] != NUM_P && red_bar == 3) {
						team = 2;
						blink = 1;
						red_bar = 0;
					}
					#endif

					hide_counter = 1000;
					busy_counter = 100000;
					key_action_counter++;
					if (key_action_counter == 300) { play_loop = 0; }
				}
				else { key_action_counter = 0; }

				// Team LED
				if (team_blink_counter > 128) { team_blink_counter = 0; }
				team_blink_counter++;

				#if (HARDWARE == 0)
				if (team_blink_counter == 120) {
					PORTD &= TEAM_MASK;
					PORTD |= (team<<1);
				}
				if (team_blink_counter == 64 && red_bar > 0) {	
					PORTD &= RECEIVE_MASK;
					DDRD |= RECEIVE;
				}
				#endif

				#if (HARDWARE == 1)
				if (team_blink_counter == 80) {
					PORTD &= TEAM_MASK;
					PORTD |= (team<<1);
				}
				#endif

				if (team_blink_counter == 128) {	
					PORTD &= TEAM_MASK;	
				}
				if (team_blink_counter == 65 && red_bar > 0) {
					DDRD &= RECEIVE_MASK;	
				}

				#if (HARDWARE == 1)
				// Plant
				if (plant_disarm_counter > 0) {
					plant_disarm_counter--;
					if (plant_disarm_counter == 0) {
						red_bar = 0;
					}
				}
				// Bomb Timer
				if (bomb_timer > 0 && team != 2) {
					bomb_counter++;
					if (bomb_counter == 108) {
						bomb_counter = 0;
						hide_counter = 1000;
						bomb_timer -= 1.0;
						// Bomb Explode
						if (bomb_timer < 1.0) {
							Val_To_Seg (bomb_timer);
							#if (SOUND_ENABLE == 1)
							snd_c4_explode_pointer = 0;
							#endif	
							seg_value[0] = NUM_HYPHEN;
							seg_value[1] = NUM_HYPHEN;
							blink = 1;
							red_bar = 0;
							Fire ();
						}
						else {
							if (bomb_timer < 36.00) {
								Val_To_Seg (bomb_timer);
								#if (SOUND_ENABLE == 1)
								snd_c4_beep_pointer = 0;
								#endif
								fire_flash_counter = 16;
							}
						}
					}
					else {
						#if (SOUND_ENABLE == 1)
						if (bomb_timer < 11.0 && bomb_timer > 3.0 && bomb_counter == 54) {
							snd_c4_beep_pointer = 0;						
						}
						if (bomb_timer < 4.0 && (bomb_counter == 27 || bomb_counter == 54 || bomb_counter == 81)) {
							snd_c4_beep_pointer = 0;						
						}
						#endif
					}
				}
				#endif

				// Not Playing
				if (busy_counter == 0) {
					play_loop = 0;
				}
			}
		}
	}
	return 0;
}

// ########## Initialisierung ##########

void Init (void) {
	// Disable JTAG
	MCUCSR |= 0x80;
	MCUCSR |= 0x80;

	// Port A
	// 0x01 7SEG /A (OUT)
	// 0x02 7SEG /B (OUT)
	// 0x04 7SEG /C (OUT)
	// 0x08 7SEG /D (OUT)
	// 0x10 7SEG /E (OUT)
	// 0x20 7SEH /F (OUT)
	// 0x40 7SEG /G (OUT)
	// 0x80 LOW BATTERY (IN)
	DDRA = 0x7F;
	PORTA = 0x7F;

	// Port B
	// 0x01 AMMO1 (OUT)
	// 0x02 AMMO10 (OUT)
	// 0x04 /HP1 (OUT)
	// 0x08 /HP2 (OUT)
	// 0x10 /HP3 (OUT)
	DDRB = 0xFF;
	PORTB = 0x1C;

	// Port C
	//8 Bit AUDIO (OUT)
	DDRC = 0xFF;
	PORTC = 0x80;

	// Port D
	// 0x01 /WHITE LED (OUT)
	// 0x02 TEAM RED (OUT)
	// 0x04 TEAM GREEN (OUT)
	// 0x08 ACTION (IN)
	// 0x10 RELOAD (IN)
	// 0x20 FIRE (IN)
	// 0x40 RECEIVE (IN)
	// 0x80 IR-LED (OUT)
	DDRD = 0x87;
	PORTD = 0x79;

	// Init Timer 1
	TCCR1A |= 0x00;
	TCCR1B |= ((1<<WGM12)|(0<<CS12)|(0<<CS11)|(1<<CS10));	// Prescaler 1 und..
	OCR1A = 211;												// Output Compare Register A (16 MHz / 211 = 75,82939 kHz)
	TIMSK |= (1<<OCIE1A);										// Interrupt Timer erlauben

	// Init ADC
	SFIOR = (0<<ADTS2)|(0<<ADTS1)|(0<<ADTS0)|(0<<ACME)|(0<<PUD)|(0<<PSR2)|(0<<PSR10);
	ADCSRA = (1<<ADEN)|(1<<ADSC)|(0<<ADATE)|(1<<ADIF)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
	ADMUX  = (0<<REFS1)|(1<<REFS0)|(0<<ADLAR)|(0<<MUX3)|(1<<MUX2)|(1<<MUX1)|(1<<MUX0);

	// Enable Interrupts
	sei();
}

// ########## Intro ##########

void Intro (void) {	
unsigned char intro_seg[] = { 0x7F, 0x7E, 0x7D, 0x7B, 0X77, 0x6F, 0x5F, 0x7E, 0x7D, 0x7B, 0X77, 0x6F, 0x5F, 0x3F };
unsigned char intro_loop = 0;
	hide_counter = 1000;
	busy_counter = 100000;
	blink = 0;
	seg_value[0] = 0x7F;
	seg_value[1] = 0x7F;
	wait_flag = 0;
	while (wait_flag < 4) { }
	for (intro_loop = 0; intro_loop < 14; intro_loop++) {
		wait_flag = 0;
		while (wait_flag < 4) { }
		seg_value[1] = intro_seg[intro_loop];
		seg_value[0] = intro_seg[intro_loop];	
	}
	wait_flag = 0;
	while (wait_flag < 4) { }
	PORTD &= WHITE_LED_MASK;
	wait_flag = 0;
	while (wait_flag < 4) { }
	PORTD |= WHITE_LED;
}


// ########## Select Team / Charge Control ##########

void Select_Charge (void) {	
unsigned char key;
unsigned char team_select = 0;
double voltage = 0.0;
double voltage_top = 0.0;
double voltage_low = 100.0;
int delta_counter = 0;
int voltage_counter = 0;
int voltage_div = 0;
	blink = 0;
	while (1) {
		if (timer_flag) {
			timer_flag = 0;
			// Charge Control
			hide_counter = 1000;
			busy_counter = 100000;
			voltage_div++;
			if (voltage_div == 9999) {
				voltage_div = 0;
				voltage += (adc_value / 6.82667);
				voltage_counter++;
				if (voltage_counter == 99) {
					voltage_counter = 0;
					voltage /= 100;
					Val_To_Seg (voltage);
					if (voltage_top < voltage) {
						voltage_top = voltage;
						team = 2;
						delta_counter = 0;
					}
					else {
						if (voltage > 65.0) {
							delta_counter++;
							team = 1;
						}
					}
					if (voltage_low > voltage && delta_counter < 3) {
						voltage_low = voltage;
						team = 0;
						delta_counter = 0;
					}
					voltage = 0;
				}
				if (voltage_counter == 3 || voltage_counter == 53) {
					if (delta_counter <= 2) {
						red_bar = 0;
						if (team_select == 0) {
							seg_value[0] = NUM_BLANK;
							seg_value[1] = NUM_BLANK;
							team = 0;
						}
						else {
							team_select = 0;
						}
					}
					if (delta_counter == 3) {
						red_bar = 1;
					}
					if (delta_counter == 4) {
						red_bar = 2;
					}
					if (delta_counter == 5) {
						red_bar = 3;
					}
					if (delta_counter > 5) {
						red_bar = 3;
						delta_counter = 6;
						if (team_select == 0) {
							team = 3;
						}
						else {
							team_select = 0;
						}
					}
				}
				if (voltage_counter == 50) {
					seg_value[0] = NUM_A;
					seg_value[1] = NUM_C;
				}
			}
		}

		#if (HARDWARE == 0)
		// Select Team
		key = Get_Key ();
		if (key == KEY_FIRE && team > 0) { return; }
		if (key == KEY_ACTION) {
			#if (SOUND_ENABLE == 1)
			snd_button_pointer = 0;
			#endif
			voltage_div = 0;
			voltage_counter = 0;
			voltage = 0;
			team_select =  1;
			seg_value[0] = NUM_5;
			seg_value[1] = NUM_E;
			switch (team) {
				case 0:
					team = 1;
					break;
				case 1:
					team = 2;
					break;
				case 2:
					team = 3;
					break;
				case 3:
					team = 1;
					break;
			}
		}
		#endif
		
		#if (HARDWARE == 1)
		key = Get_Key ();
		if (key == KEY_ACTION) { return; }
		#endif

		PORTD &= TEAM_MASK;
		PORTD |= (team<<1);
	}
}

// ########## Value To Seg. ##########
void Val_To_Seg (double value) {
int s2 = 0;
int s1 = 0;
	s2 = (value / 10);
	s1 = value - (s2 * 10);
	if (value < 100) {
		switch (s2) {
			case 0: seg_value[0] = NUM_0; break;
			case 1: seg_value[0] = NUM_1; break;
			case 2: seg_value[0] = NUM_2; break;
			case 3: seg_value[0] = NUM_3; break;
			case 4: seg_value[0] = NUM_4; break;
			case 5: seg_value[0] = NUM_5; break;
			case 6: seg_value[0] = NUM_6; break;
			case 7: seg_value[0] = NUM_7; break;
			case 8: seg_value[0] = NUM_8; break;
			case 9: seg_value[0] = NUM_9; break;
		}
		switch (s1) {
			case 0: seg_value[1] = NUM_0; break;
			case 1: seg_value[1] = NUM_1; break;
			case 2: seg_value[1] = NUM_2; break;
			case 3: seg_value[1] = NUM_3; break;
			case 4: seg_value[1] = NUM_4; break;
			case 5: seg_value[1] = NUM_5; break;
			case 6: seg_value[1] = NUM_6; break;
			case 7: seg_value[1] = NUM_7; break;
			case 8: seg_value[1] = NUM_8; break;
			case 9: seg_value[1] = NUM_9; break;
		}
	}
	else {
		seg_value[0] = NUM_F;
		seg_value[1] = NUM_U;
	}
}

// ########## Get Key ##########

unsigned char Get_Key () {
unsigned char key_action;
unsigned char key_reload;
unsigned char key_fire;
	key_action = PIND & KEY_ACTION;
	key_reload = PIND & KEY_RELOAD;
	key_fire   = PIND & KEY_FIRE;
	if (key_wait) {
		if (key_action == 0 || key_reload == 0 || key_fire == 0) { return 0; }
		key_wait--;
		return 0;
	}
	if (key_action == 0) {
		key_wait = 5999;
		return KEY_ACTION;
	}
	if (key_reload == 0) {
		key_wait = 5999;
		return KEY_RELOAD;
	}
	if (key_fire == 0) {
		key_wait = 5999;
		return KEY_FIRE;
	}
	return 0;
}

// ########## Fire ##########

void Fire () {
	#if (HARDWARE == 0)
	if (red_bar == 0) { return; }
	if (reload_counter > 0) { return; }
	if (fire_flash_counter > 0) { return; }
	send_buffer[1] = team;
	if (seg_value[0] != NUM_C) {
		if (ammo == 0) {
			#if (SOUND_ENABLE == 1)
			snd_empty_pointer = 0;
			#endif
			return;
		}
		send_buffer[2] = 1;
		fire_flash_counter = 16;
		ammo--;
		Set_Ammo ();
		#if (SOUND_ENABLE == 1)
		snd_fire_pointer = 0;
		#endif
	}
	else {
		send_buffer[2] = 0;
		fire_flash_counter = 128;
		#if (SOUND_ENABLE == 1)
		snd_button_pointer = 0;
		#endif
	}
	#endif

	#if (HARDWARE == 1)
	send_buffer[1] = 3;
	send_buffer[2] = 1;
	fire_flash_counter = 256;
	#endif
	
	send_buffer[3] = 3 - ((send_buffer[1] + send_buffer[2] +1) & 0X03);
	send_pointer = 0;

	#if (HARDWARE == 0)
	send_repeat = 3;
	#endif

	#if (HARDWARE == 1)
	send_repeat = 30;
	#endif
}

// ########## Chk Hit / Disarm / Plant ##########

void Chk_Receive () {
unsigned char rec_byte[] = { 0, 0, 0 };
	if (rec_flag != 1) { return; }
	rec_flag = 0;
	if (rec_buffer[0] != 1) { return; }

	#if (HARDWARE == 0)
	if (red_bar == 0) { return; }
	#endif

	rec_byte[0] = rec_buffer[1];
	rec_byte[1] = rec_buffer[2];
	rec_byte[2] = rec_buffer[3];

	// Cheksum
	if  (3 - ((rec_byte[0] + rec_byte[1] +1) & 0X03) != rec_byte[2]) { return; }

	#if (HARDWARE == 0)
	// TEAM (1=RED, 2=GREEN, 3=DEATHMATCH)
	if (team == 3 && rec_byte[0] != 3) { return; }
	if (team == 1 && rec_byte[0] != 2 && rec_byte[0] != 3) { return; }
	if (team == 2 && rec_byte[0] != 1 && rec_byte[0] != 3) { return; }
	#endif

	#if (HARDWARE == 0)
	// ACTION (1=FIRE)
	if (rec_byte[1] != 1) { return; }
	#endif

	#if (HARDWARE == 1)
	// ACTION (0=DISARM/PLANT)
	if (rec_byte[1] != 0) { return; }
	#endif

	hide_counter = 1000;
	busy_counter = 100000;

	#if (HARDWARE == 0)
	// HIT
	if (red_bar > 0) {
		red_bar--;
		hit_pause = 9999;
		#if (SOUND_ENABLE == 1)
		if (red_bar > 0) { snd_autschn_pointer = 0; }
		#endif
	}
	if (red_bar == 0) {
		death_counter = 2000;
		blink = 1;
		seg_value[0] = NUM_d;
		seg_value[1] = NUM_BLANK;
		PORTD &= RECEIVE_MASK;
		DDRD |= RECEIVE;
		#if (SOUND_ENABLE == 1)
		snd_manno_pointer = 0;
		#endif
	}
	#endif

	#if (HARDWARE == 1)
	if (rec_byte[0] == 1 && seg_value[0] == NUM_P) {
		if (red_bar < 3) {
			red_bar++;
			plant_disarm_counter = 500;
		}
		hit_pause = 9999;
	}
	if (rec_byte[0] == 2 && bomb_timer > 0.0 && team == 1) {
		if (red_bar < 3) {
			red_bar++;
			plant_disarm_counter = 500;
		}
		hit_pause = 9999;
	}
	#endif
}

// ########## Set Ammo ##########

#if (HARDWARE == 0)
void Set_Ammo (void) {
	switch (ammo) {
		case 0:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_0; break;
		case 1:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_1; break;
		case 2:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_2; break;
		case 3:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_3; break;
		case 4:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_4; break;
		case 5:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_5; break;
		case 6:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_6; break;
		case 7:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_7; break;
		case 8:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_8; break;
		case 9:  seg_value[0] = NUM_BLANK; seg_value[1] = NUM_9; break;
		case 10: seg_value[0] = NUM_1;     seg_value[1] = NUM_0; break;
		case 11: seg_value[0] = NUM_1;     seg_value[1] = NUM_1; break;
		case 12: seg_value[0] = NUM_1;     seg_value[1] = NUM_2; break;
	}
}
#endif

// ########## Interrupt ADC ##########

ISR (ADC_vect) {
int adc_l = 0;
int adc_h = 0;
	adc_l = ADCL;
	adc_h = ADCH;
	adc_value = (adc_h << 8) + adc_l;
	// Start next ADC Sampling
	ADMUX  = (0<<REFS1)|(1<<REFS0)|(0<<ADLAR)|(0<<MUX3)|(1<<MUX2)|(1<<MUX1)|(1<<MUX0);
	ADCSRA = (1<<ADEN)|(1<<ADSC)|(0<<ADATE)|(1<<ADIF)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
}

// ########## Interrupt Timer 1 ##########

ISR (TIMER1_COMPA_vect) {
	timer_flag++;

	// IR-LED SEND
	if (send_pointer < MAX_SEND_BITS) {
		if (ir_led_counter < 64) { 	PORTD ^= IR_LED; }
		else { PORTD &= IR_LED_MASK; }
		ir_led_counter++;
		if ((send_buffer[send_pointer] == 0 && ir_led_counter == 128) || (send_buffer[send_pointer] == 1 && ir_led_counter == 256) || (send_buffer[send_pointer] == 2 && ir_led_counter == 384) || (send_buffer[send_pointer] == 3 && ir_led_counter == 512)) {
			ir_led_counter = 0;
			send_pointer++;
			send_pause = 999;
		}
	}
	else {
		if (send_pause > 0) {
			send_pause--;
		}
		else {
			if (send_repeat > 0) {
				send_repeat--;
				send_pointer = 0;
			}
		}
	}

	// IR RECEIVE
	if (hit_pause > 0) {
		hit_pause--;
	}
	else {
		if (ir_rec_counter < 999) { ir_rec_counter++; }
		if ((PIND & RECEIVE)) { ir_low_rising = 0; }
		else {
			if (ir_low_rising == 0) {
				if (ir_rec_counter == 999) { rec_pointer = 0; }
				ir_low_rising = 1;
				if (ir_rec_counter != 999 && ir_rec_counter > 64) {
					if (ir_rec_counter > 448) { rec_buffer[rec_pointer] = 3; }
					else {
						if (ir_rec_counter > 320) { rec_buffer[rec_pointer] = 2; }
						else {
							if (ir_rec_counter > 192) { rec_buffer[rec_pointer] = 1; }
							else {
								rec_buffer[rec_pointer] = 0;
							}
						}
					}
					if (rec_pointer < MAX_REC_BITS) {
						rec_pointer++;
						if (rec_pointer == MAX_REC_BITS) {
							rec_flag = 1;
						}
					}
				}
				ir_rec_counter = 0;
			}
		}
	}

	// 7-SEG + RED LEDS
	seg_div++;
	if (seg_div == 700) {
		seg_div = 0;
		wait_flag++;
		if (hide_counter > 0) { hide_counter--; }
		if (busy_counter > 0) { busy_counter--; }
		PORTB |= AMMO1 + AMMO10;
		blink_counter++;
		if (blink_counter > 64) { blink_counter = 0; }
		if (hide_counter != 0 && (blink == 0 || blink_counter > 48)) {
			if (seg_pos == AMMO1) {
				PORTA = SEG_MASK;
				PORTB &= (AMMO_MASK + AMMO1);
				PORTA = seg_value[0];
				seg_pos = AMMO10;
			}
			else {
				PORTA = SEG_MASK;
				PORTB &= (AMMO_MASK + AMMO10);
				PORTA = seg_value[1];
				seg_pos = AMMO1;
			}
		}
		// HP
		if (hide_counter != 0) { 
			PORTB |= RED_MASK;
			switch (red_bar) {
				case 3:
					PORTB &= ~(RED1_LED + RED2_LED + RED3_LED); 
					break;
				case 2:
					PORTB &= ~(RED1_LED + RED2_LED);
					break;
				case 1:
					PORTB &= ~RED1_LED;
					break;
			}
		}
		else { PORTB |= RED_MASK; }
	}

	// Sound
	#if (SOUND_ENABLE == 1)
	snd_div--;
	if (!snd_div) {
		snd_div = 9;

		#if (HARDWARE == 0)
		if (snd_fire_pointer < snd_fire_stop-1) {
			snd_fire_pointer++;
			snd_data = pgm_read_byte(&(snd_fire[snd_fire_pointer]));
			PORTC = snd_data;
		}
		else {
			if (snd_autschn_pointer < snd_autschn_stop-1) {
				snd_autschn_pointer++;
				snd_data = pgm_read_byte(&(snd_autschn[snd_autschn_pointer]));
				PORTC = snd_data;
			}
			if (snd_manno_pointer < snd_manno_stop-1) {
				snd_manno_pointer++;
				snd_data = pgm_read_byte(&(snd_manno[snd_manno_pointer]));
				PORTC = snd_data;
			}
		}
		if (snd_button_pointer < snd_button_stop-1) {
			snd_button_pointer++;
			snd_data = pgm_read_byte(&(snd_button[snd_button_pointer]));
			PORTC = snd_data;
		}
		if (snd_clip_pointer < snd_clip_stop-1) {
			snd_clip_pointer++;
			snd_data = pgm_read_byte(&(snd_clip[snd_clip_pointer]));
			PORTC = snd_data;
		}
		if (snd_empty_pointer < snd_empty_stop-1) {
			snd_empty_pointer++;
			snd_data = pgm_read_byte(&(snd_empty[snd_empty_pointer]));
			PORTC = snd_data;
		}
		if (snd_release_pointer < snd_release_stop-1) {
			snd_release_pointer++;
			snd_data = pgm_read_byte(&(snd_release[snd_release_pointer]));
			PORTC = snd_data;
		}
		#endif

		#if (HARDWARE == 1)
		if (snd_c4_beep_pointer < snd_c4_beep_stop-1) {
			snd_c4_beep_pointer++;
			snd_data = pgm_read_byte(&(snd_c4_beep[snd_c4_beep_pointer]));
			PORTC = snd_data;
		}
		if (snd_c4_explode_pointer < snd_c4_explode_stop-1) {
			snd_c4_explode_pointer++;
			snd_data = pgm_read_byte(&(snd_c4_explode[snd_c4_explode_pointer]));
			PORTC = snd_data;
		}
		if (snd_bomb_planted_pointer < snd_bomb_planted_stop-1) {
			snd_bomb_planted_pointer++;
			snd_data = pgm_read_byte(&(snd_bomb_planted[snd_bomb_planted_pointer]));
			PORTC = snd_data;
		}
		#endif
	}
	#endif
}
